import javax.realtime.*;

/**	Demonstrate a passive miss handler */
public class PassiveMissHdlr {
  /**	Define the passive AEH for misses */
  public static class MissHdlr extends AsyncEventHandler {
    PeriodicThread th;  //  Reference to the client thread

    public void setThread(PeriodicThread th) {
      this.th = th;
    }

    MissHdlr() {
      super(
          new PriorityParameters(
              PriorityScheduler.getMinPriority(null)+11),
          null, null, null, null, null);
    }

    public void handleAsyncEvent() {
      System.out.println("Recovering from a miss");
      //	First interact with whatever is bothered
      //	by us missing the deadline,
      //  <some action>
      th.schedulePeriodic();  //  Let the thread continue
    }
  }

  /**	Define a simple periodic RT thread */
  public static class PeriodicThread extends RealtimeThread {
    volatile double f;

    public PeriodicThread(SchedulingParameters sched,
        ReleaseParameters release) {
      super(sched, release);
    }

    public void run() {
      final int CYCLES = 15;
      int bound = 0;

      for (int ctr = 0; ctr < CYCLES; ++ctr) {
        for (f=0.0; f < bound; f += 1.0);  // Use some time
        bound += 800000;
        System.out.println("Ding! " + bound);
        waitForNextPeriod();
      }
    }
  }

  public static void main(String [] args) {
    //	Build parameters for construction of RT thread
    MissHdlr missHdlr = new MissHdlr();
    ReleaseParameters release =
      new PeriodicParameters(
          new RelativeTime(),        //	Start at .start()
          new RelativeTime(1000, 0), // 1 second period
          null,                      //	cost
          new RelativeTime(500,0),   // deadline=period/2
          null,                      // no overrun handler
          missHdlr);                 // miss handler
    SchedulingParameters scheduling = new PriorityParameters(
        PriorityScheduler.getMinPriority(null)+10);

    PeriodicThread rt= new PeriodicThread(scheduling, release);
    //	Give the miss handler a reference to
    //	the thread it is managing.
    missHdlr.setThread(rt);

    rt.start();   //	Start the periodic thread
    try {
      rt.join();  //	Wait for the thread to end
    } catch (InterruptedException e) {
      //  Ignore
    };
  }
}