import javax.realtime.*;

/**	Demonstrate a sudden-death overrun handler 
 * 	The cost-overrun detection does not work on the 1.0.2
 * 	reference implementation, but it does work on the
 * 	Alpha RTSJ version 1.1 RI.
 */
public class SuddenDeath {
  /**	Define the AEH for overruns */
  public static class OverrunHandler extends AsyncEventHandler {
    RealtimeThread th;

    public void setThread(RealtimeThread th) {
      this.th = th;
    }

    OverrunHandler() {
      super(
          new PriorityParameters(
              PriorityScheduler.getMinPriority(null)+11),
          null, null, null, null, null);
    }

    public void handleAsyncEvent() {
      System.out.println("Zapping thread that's over budget");
      th.interrupt(); //	Throw an AIE into the thread
    }
  }
  /** 
  *     Define a RT thread that stops quickly in case of an AIE.
  */
  public static class PeriodicThread extends RealtimeThread {
    volatile double f;
    public PeriodicThread(SchedulingParameters sched,
        ReleaseParameters release) {
      super(sched, release);
    }

    public void run() {
      try {
        doWork(); //	call an interruptible method
      } catch (AsynchronouslyInterruptedException e) {
        System.out.println("Thread dying");
      }
    }

    private void doWork() throws 
        AsynchronouslyInterruptedException {
      int bound=0;

      while (true) {
        do {
          // Use some time
          for (f=0.0; f < bound; f += 1.0);  
          bound += 100000;  // Use more next time
          System.out.println("Ding! " + bound);
        } while (waitForNextPeriod());
        //	Recover from miss 
        System.out.println("Scheduling error");
        bound -= 150000;  //	Lighten load
        while (!waitForNextPeriod()) //	Eat errors
          System.out.print(".");
        System.out.println();
      }
    }
  }

  public static void main(String [] args) {
    //	Build parameters for construction
    //	of RT thread 
    OverrunHandler overrunHandler = new OverrunHandler();
    ReleaseParameters release =
      new PeriodicParameters(
          new RelativeTime(),        //	Start at .start()
          new RelativeTime(1000, 0), // 1 second period
          new RelativeTime(50, 0),   //	cost
          new RelativeTime(100,0),   // deadline=period/10
          overrunHandler,            // overrun handler
          null);                     // no miss handler
    SchedulingParameters scheduling =
      new PriorityParameters(
          PriorityScheduler.getMinPriority(null)+10); 

    RealtimeThread rt= new PeriodicThread(scheduling,
        release);
    //	Give the overrun handler a reference to
    //	the thread it is managing.
    overrunHandler.setThread(rt);

    rt.start();   //	Start the periodic thread
    // we are about to cheat
    try {
      Thread.sleep(10000);
    } catch (InterruptedException e) {
    }
    AsyncEvent evt = new AsyncEvent();
    evt.addHandler(overrunHandler);
    evt.fire(); //	pretend we caught an overrun
    try {
      rt.join();  //	Wait for the thread to end
    } catch (Exception e) {
    };
  }
}
